@@ -1,6 +1,7 @@
 """This defines a basic wsgi application that provides RESTful routing."""
 
 from werkzeug.exceptions import HTTPException
+from werkzeug.exceptions import NotFound
 from werkzeug.exceptions import MethodNotAllowed
 from werkzeug.exceptions import InternalServerError
 from werkzeug.wrappers import Request
@@ -45,21 +46,48 @@ class RestApplication(object):
         # variable is captialzied.
         # This call automatically raises a `NotFound` exception if there is no
         # match found.
-        Endpoint, kwargs = urls.match()
+        try:
+
+            Endpoint, kwargs = urls.match()
+
+        except NotFound as e:
+
+            response = e
+
+            self.process_response_hooks(response, response)
+
+            return response
 
         handler_endpoint = Endpoint()
 
+        # There is some potential that this exception should return a
+        # NotImplemented rather than a MethodNotAllowed. The
+        # MethodNotAllowed
         try:
 
             handler_method = getattr(handler_endpoint, verb)
 
         except AttributeError:
 
-            raise MethodNotAllowed()
+            # If method not found then generate a list of allowed methods.
+            # The list of allowed methods is required to satisfy the HTTP
+            # standard for Method Now Allowed. Currently only matches GET,
+            # POST, PUT, and DELETE when producing a list.
+            response = MethodNotAllowed(valid_methods=[m for m
+                                                       in dir(handler_endpoint)
+                                                       if m
+                                                       in ['GET',
+                                                           'POST',
+                                                           'PUT',
+                                                           'DELETE'
+                                                           ]
+                                                       ])
 
-        for hook in self.pre_hooks:
+            self.process_response_hooks(request, response)
 
-            hook(request)
+            return response
+
+        self.process_request_hooks(request)
 
         try:
 
@@ -73,8 +101,18 @@ class RestApplication(object):
 
             response = InternalServerError(description=unicode(e))
 
+        self.process_response_hooks(request, response)
+
+        return response
+
+    def process_request_hooks(self, request):
+
+        for hook in self.pre_hooks:
+
+            hook(request)
+
+    def process_response_hooks(self, request, response):
+
         for hook in self.post_hooks:
 
             hook(request, response)
-
-        return response
